#include "board.h"
#include "7seg-common.h"

uint32_t display[NDIGITS];
uint32_t display_fade[NDIGITS];

// encode table for digits 0..9
static const uint8_t encode_digit[] = {
	SEG_A_MSK | SEG_B_MSK | SEG_C_MSK | SEG_D_MSK | SEG_E_MSK | SEG_F_MSK,	// 0
	SEG_B_MSK | SEG_C_MSK,	// 1
	SEG_A_MSK | SEG_B_MSK | SEG_D_MSK | SEG_E_MSK | SEG_G_MSK,	// 2
	SEG_A_MSK | SEG_B_MSK | SEG_C_MSK | SEG_D_MSK | SEG_G_MSK,	// 3
	SEG_B_MSK | SEG_C_MSK | SEG_F_MSK | SEG_G_MSK,	// 4
	SEG_A_MSK | SEG_C_MSK | SEG_D_MSK | SEG_F_MSK | SEG_G_MSK,	// 5
	SEG_A_MSK | SEG_C_MSK | SEG_D_MSK | SEG_E_MSK | SEG_F_MSK | SEG_G_MSK,	// 6
	SEG_A_MSK | SEG_B_MSK | SEG_C_MSK,	// 7
	SEG_A_MSK | SEG_B_MSK | SEG_C_MSK | SEG_D_MSK | SEG_E_MSK | SEG_F_MSK | SEG_G_MSK,	// 8
	SEG_A_MSK | SEG_B_MSK | SEG_C_MSK | SEG_D_MSK | SEG_F_MSK | SEG_G_MSK	// 9
};
static const uint16_t digmask[] = {DIG0_MSK, DIG1_MSK, DIG2_MSK, DIG3_MSK};

// set clock to 80 MHz, PLL MSI-fed, sychronized to LSE
// setup GPIOC
void common_setup(void)
{
	RCC->APB1ENR1 = RCC_APB1ENR1_PWREN | RCC_APB1ENR1_TIM4EN;
	RCC->AHB2ENR = RCC_AHB2ENR_GPIOCEN;
	PWR->CR1 |= PWR_CR1_DBP;	// Enable access to BDCR
	RCC->BDCR = RCC_BDCR_LSEON;
	// GPIOC - display and button
	GPIOC->MODER = BF2A(0, GPIO_MODER_OUT) & BF2A(1, GPIO_MODER_OUT) & BF2A(2, GPIO_MODER_OUT) & BF2A(3, GPIO_MODER_OUT)
		 & BF2A(4, GPIO_MODER_OUT) & BF2A(5, GPIO_MODER_OUT) & BF2A(6, GPIO_MODER_OUT) & BF2A(7, GPIO_MODER_OUT)
		 & BF2A(8, GPIO_MODER_OUT) & BF2A(9, GPIO_MODER_OUT) & BF2A(10, GPIO_MODER_OUT) & BF2A(11, GPIO_MODER_OUT)
		 & BF2A(13, GPIO_MODER_IN);
	GPIOC->PUPDR = BF2(13, GPIO_PUPDR_PU);	// Nucleo64 button
	while (~RCC->BDCR & RCC_BDCR_LSERDY);
	RCC->CR |= RCC_CR_MSIPLLEN;	// Sync MSI to LSE
	// PLL - 80 MHz
	RCC->PLLCFGR = RCC_PLLCFGR_PLLREN | RCC_PLLCFGR_PLLNV(40) | RCC_PLLCFGR_PLLMV(1) | RCC_PLLCFGR_PLLSRC_MSI;
	RCC->CR |= RCC_CR_PLLON;	//
	// set Flash speed
	FLASH->ACR = FLASH_ACR_DCEN | FLASH_ACR_ICEN | FLASH_ACR_PRFTEN | FLASH_ACR_LATENCY_4WS;	// 4ws 64..80
	while (!(RCC->CR & RCC_CR_PLLRDY));
	RCC->CFGR |= RCC_CFGR_SW_PLL;
}

// key handling, brightness control, time increment
void run_every_10ms(void)
{
	static uint8_t k_nuc_hist;
	static uint8_t brightness_target = MPX_BRIGHT;
	
	// key handling
	if ((k_nuc_hist = (k_nuc_hist << 1 | BTN_DOWN) & 3) == 1)
		brightness_target ^= MPX_BRIGHT ^ MPX_DIM;
	
	// brightness adjust
	uint32_t d = LEDMpx_TIM->CCR1;
	if (d != brightness_target)
		LEDMpx_TIM->CCR1 = d < brightness_target ? d + 1 : d - 1;
	
	static uint16_t tdiv = 99;
	if (++ tdiv == 100)
	{
		tdiv = 0;
		// 1s time increment
		static uint16_t seccnt = 9999;
		if (++ seccnt == 10000)
			seccnt = 0;
		// prepare image
		uint32_t v = seccnt;
		for (uint32_t i = 0; i < NDIGITS; i ++)
		{
			display_fade[i] = display[i];
			display[i] = DigAct(digmask[i]) | SegAct(v || !i ? encode_digit[v % 10] : 0);
			v /= 10;
		}
		LEDMpx_TIM->CCR2 = 1;
#ifdef DMAFADE
		LEDMpx_TIM->DIER = TIM_DIER_CC2DE | TIM_DIER_CC1DE | TIM_DIER_UDE | TIM_DIER_UIE;
#else
		LEDMpx_TIM->DIER = TIM_DIER_CC2IE | TIM_DIER_CC1IE | TIM_DIER_UIE;
#endif
	}
}
