/*
STM32L476 + Nucleo Companion Board
7-segment multiplexed display, soft refresh w/fading
gbm, 01'2017
*/

#include "board.h"
#include "7seg-common.h"

int main(void)
{
	// setup clock & GPIOC
	common_setup();
	// LED multiplexing timer
	LEDMpx_TIM->PSC = SYSCLK_FREQ / MPX_FREQ / MPX_STEPS - 1;
	LEDMpx_TIM->ARR = MPX_STEPS - 1;
	LEDMpx_TIM->DIER = TIM_DIER_CC1IE | TIM_DIER_UIE;
	LEDMpx_TIM->CR1 = TIM_CR1_CEN;
	
	NVIC_EnableIRQ(LEDMpx_IRQn);
	SCB->SCR = SCB_SCR_SLEEPONEXIT_Msk;	// sleep while not in handler
	__WFI();	// go to sleep
}

void LEDMpx_IRQHandler(void)
{
	static uint8_t dig;
	uint32_t tim_sr = LEDMpx_TIM->SR & LEDMpx_TIM->DIER;
	LEDMpx_TIM->SR = ~(TIM_SR_UIF | TIM_SR_CC1IF | TIM_SR_CC2IF);
	if (tim_sr & TIM_SR_UIF)
	{
		if (dig == 0
			&& (LEDMpx_TIM->DIER & TIM_DIER_CC2IE)
			&& ++ LEDMpx_TIM->CCR2 >= LEDMpx_TIM->CCR1)
			LEDMpx_TIM->DIER = TIM_DIER_CC1IE | TIM_DIER_UIE;

		dig = (dig + 1) % NDIGITS;
		LEDMpx_PORT->BSRR = display[dig];
		
		static uint8_t tdiv;
		if (++ tdiv == MPX_FREQ / 100)
		{
			tdiv = 0;
			run_every_10ms();
		}
	}
	if (tim_sr & TIM_SR_CC1IF)
		LEDMpx_PORT->BSRR = DigAct(0) | SegAct(0);
	if (tim_sr & TIM_SR_CC2IF)
		LEDMpx_PORT->BSRR = display_fade[dig];
}
