/*
STM32L476 + Nucleo Companion Board
7-segment mpx display, soft refresh, brightness control
gbm, 01'2017
*/

#include "board.h"
#include "7seg-common.h"

int main(void)
{
	common_setup();	// setup clock & GPIOC
	// LED multiplexing timer
	LEDMpx_TIM->PSC = SYSCLK_FREQ / MPX_FREQ / MPX_STEPS - 1;
	LEDMpx_TIM->ARR = MPX_STEPS - 1;
	LEDMpx_TIM->DIER = TIM_DIER_CC1IE | TIM_DIER_UIE;
	LEDMpx_TIM->CR1 = TIM_CR1_CEN;
	// interrupts and sleep control
	NVIC_EnableIRQ(LEDMpx_IRQn);
	SCB->SCR = SCB_SCR_SLEEPONEXIT_Msk;	// sleep while not in handler
	__WFI();	// go to sleep
}

void LEDMpx_IRQHandler(void)
{
	uint32_t tim_sr = LEDMpx_TIM->SR;
	LEDMpx_TIM->SR = ~(TIM_SR_UIF | TIM_SR_CC1IF);
	
	if (tim_sr & TIM_SR_UIF)
	{
		static uint8_t dig;
		dig = (dig + 1) % NDIGITS;
		LEDMpx_PORT->BSRR = display[dig];
		
		static uint8_t tdiv;
		if (++ tdiv == MPX_FREQ / 100)
		{
			tdiv = 0;
			run_every_10ms();
		}
	}
	if (tim_sr & TIM_SR_CC1IF)
		LEDMpx_PORT->BSRR = DigAct(0) | SegAct(0);
}
